#pragma once

#include <filesystem>

#include "../../../../core/error.h"

namespace armarx::armem::server::ltm::disk
{
    namespace constantes
    {
        const std::string TYPE_FILENAME = "type.aron";
        const std::string DATA_FILENAME = "data.aron";
        const std::string METADATA_FILENAME = "metadata.aron";
    }

    namespace util
    {
        // check whether a string is a number (e.g. timestamp folders)
        inline bool isNumber(const std::string& s)
        {
            for (char const& ch : s)
            {
                if (std::isdigit(ch) == 0)
                {
                    return false;
                }
            }
            return true;
        }

        inline bool checkIfFolderExists(const std::filesystem::path& p)
        {
            return std::filesystem::exists(p) and std::filesystem::is_directory(p);
        }

        inline void ensureFolderExists(const std::filesystem::path& p, bool createIfNotExistent = true)
        {
            if (!std::filesystem::exists(p))
            {
                if (createIfNotExistent)
                {
                    std::filesystem::create_directories(p);
                }
            }
            if (!std::filesystem::is_directory(p))
            {
                throw error::ArMemError("Could not find folder: " + p.string());
            }
        }

        inline bool checkIfFileExists(const std::filesystem::path& p)
        {
            return std::filesystem::exists(p) and std::filesystem::is_regular_file(p);
        }

        inline void ensureFileExists(const std::filesystem::path& p)
        {
            if (!std::filesystem::exists(p) || !std::filesystem::is_regular_file(p))
            {
                // not found
                throw error::ArMemError("Could not find file: " + p.string());
            }
        }
    }
} // namespace armarx::armem::server::ltm::disk
