// Header
#include "DiskStorage.h"

// ArmarX
#include <ArmarXCore/core/time/TimeUtil.h>
#include <ArmarXCore/core/logging/Logging.h>

namespace armarx::armem::server::ltm::disk
{
    DiskMemoryItem::DiskMemoryItem(const std::filesystem::path& p) :
        path(p.string())
    {
    }

    bool DiskMemoryItem::checkPath(const std::string& suffix) const
    {
        std::filesystem::path memoryP(path);
        memoryP = memoryP / suffix;

        // Check connection:
        if (!std::filesystem::exists(path))
        {
            std::filesystem::create_directories(path);
            return true;
        }
        else if (std::string expectedFolderName = getExpectedFolderName(); !std::filesystem::is_directory(path) || memoryP.filename() != expectedFolderName)
        {
            ARMARX_WARNING << deactivateSpam("LTM_PathError_" + expectedFolderName)
                           << "The entered path is not valid. Please use a path leading to a memory folder with name: " << expectedFolderName << ". The used path was: " << getPath()
                           << "\n\n";
            return false;
        }

        return true;
    }

    std::string DiskMemoryItem::getPath() const
    {
        return path;
    }

    void DiskMemoryItem::setPath(const std::string& ps)
    {
        path = ps;
    }
}
