#include "SkillExecutionRequestSegment.h"

#include <RobotAPI/libraries/armem/server/MemoryToIceAdapter.h>
#include <SimoxUtility/algorithm/string.h>

#include <RobotAPI/libraries/aron/converter/json/NLohmannJSONConverter.h>

#include <RobotAPI/libraries/armem_skills/aron/Skill.aron.generated.h>


namespace armarx::skills::segment
{

    SkillExecutionRequestCoreSegment::SkillExecutionRequestCoreSegment(armem::server::MemoryToIceAdapter& iceMemory):
        Base(iceMemory, CoreSegmentName/*, skills::arondto::SkillExecutionRequest::ToAronType()*/)
    {
    }

    void SkillExecutionRequestCoreSegment::defineProperties(PropertyDefinitionsPtr defs, const std::string &prefix)
    {
        // No properties! (meaning no name and no max size)
    }

    void SkillExecutionRequestCoreSegment::init()
    {
        Base::init();
    }


    skills::manager::dto::SkillExecutionInfo SkillExecutionRequestCoreSegment::convertCommit(const aron::data::dto::DictPtr& commitData)
    {
        // convert ice commitData to aron
        auto commitDataAron = std::make_shared<aron::data::Dict>(commitData);

        // we got a skill execution request
        skills::arondto::SkillExecutionRequest request;
        request.fromAron(commitDataAron);
        auto params = aron::data::Dict::DynamicCastAndCheck(commitDataAron->at("params")); // ToDo remov and add to request

        skills::manager::dto::SkillExecutionInfo info;
        info.providerName = request.providerName;
        info.skillName = request.skillName;
        info.params = aron::data::Dict::ToAronDictDTO(params);
        return info;
    }


    void SkillExecutionRequestCoreSegment::addSkillExecutionRequest(const skills::manager::dto::SkillExecutionInfo& info)
    {
        // override directly execution to add a request to the memory
        armem::Commit comm;
        auto& entityUpdate = comm.add();

        skills::arondto::SkillExecutionRequest request;
        request.clientId = "";
        request.providerName = info.providerName;
        request.skillName = info.skillName;

        auto aron = request.toAron();

        aron::data::DictPtr aron_params = aron::data::Dict::FromAronDictDTO(info.params);
        aron->addElement("params", aron_params); // todo add as any type

        armem::MemoryID skillExecutionMemID = id();
        skillExecutionMemID.providerSegmentName = request.providerName;
        skillExecutionMemID.entityName = request.skillName;

        entityUpdate.entityID = skillExecutionMemID;
        entityUpdate.instancesData = { aron };
        entityUpdate.confidence = 1.0;
        entityUpdate.timeCreated = armem::Time::now();

        iceMemory.commit(comm);
    }
}
