#include "Skill.h"

namespace armarx
{
    namespace skills
    {
        Skill::Skill(const SkillDescription& desc):
            description(desc)
        {
            // replace constructor if you want to have a specific logging tag
            Logging::setTag("armarx::skills::" + description.skillName);
        }

        std::thread Skill::installTimeoutCondition()
        {
            // start timeout check
            return std::thread{ [&](){
                    if (description.timeoutMs <= 0) return;
                    long skillStartedAt = IceUtil::Time::now().toMilliSeconds();
                    while(running)
                    {
                        auto now = IceUtil::Time::now().toMilliSeconds();
                        if ((now - skillStartedAt) >= description.timeoutMs) notifyTimeoutReached();
                        std::this_thread::sleep_for(std::chrono::milliseconds(100));
                    }
                }
            };
        }

        void Skill::notifyTimeoutReached()
        {
            timeoutReached = true;
            _notifyTimeoutReached();
        }

        void Skill::notifyStopped()
        {
            stopped = true;
            _notifyStopped();
        }

        void Skill::reset()
        {
            stopped = false;
            timeoutReached = false;

            //provider = nullptr;
            //manager = nullptr;

            _reset();
        }

        void Skill::_notifyTimeoutReached() { }
        void Skill::_notifyStopped() { }
        void Skill::_reset() { }

        Skill::Status Skill::execute(const aron::data::DictPtr& params, const CallbackT& callback)
        {
            running = true;
            started = IceUtil::Time::now().toMilliSeconds();

            auto timeoutCheck = installTimeoutCondition();

            auto ret = _execute(params, callback);

            started = 0;
            running = false;
            timeoutCheck.join(); // safely wait for the timeoutcheck to return
            return ret;
        }

        Skill::Status Skill::_execute(const aron::data::DictPtr& params, const CallbackT& callback)
        {
            (void) params;

            ARMARX_WARNING_S << "You have to override this method!";
            return Status::Succeeded;
        }

    }
}
