#pragma once

#include <mutex>
#include <queue>
#include <thread>
#include <functional>

#include <ArmarXCore/core/ComponentPlugin.h>
#include <ArmarXCore/core/ManagedIceObject.h>

#include <ArmarXCore/core/services/tasks/RunningTask.h>

#include <RobotAPI/interface/skills/SkillManagerInterface.h>
#include <RobotAPI/libraries/aron/core/data/variant/All.h>

#include "SkillDescription.h"

namespace armarx
{
    namespace skills
    {
        class Skill : public armarx::Logging
        {
        public:
            using CallbackT = std::function<void(const aron::data::DictPtr&)>;

            enum class Status
            {
                Succeeded,
                TimeoutReached,
                Stopped,
                Failed
            };

            Skill() = delete;
            Skill(const SkillDescription&);
            virtual ~Skill() = default;

            /// Execute Skill
            Status execute(const aron::data::DictPtr& params, const CallbackT& callback = [](const aron::data::DictPtr& returnValue) { (void) returnValue; });

            /// Reset all parameters before starting a skill.
            void reset();

            /// Set the notification bools
            void notifyTimeoutReached();
            void notifyStopped();

            /// Override this method with the actual implementation. The callback is for status updates to the calling instance
            virtual Status _execute(const aron::data::DictPtr& params, const CallbackT& callback = [](const aron::data::DictPtr& returnValue) { (void) returnValue; });

            /// Override if you have special members that needs to be resetted
            virtual void _reset();

            /// Override these methods if you want to do something special when notification comes
            virtual void _notifyTimeoutReached();
            virtual void _notifyStopped();

        private:
            /// install a condition as a seperate thread
            std::thread installTimeoutCondition();

        public:
            const SkillDescription description;

            /// running params
            std::atomic_bool running = false;
            long started = 0;

            /// proxies that called the skills
            //provider::dti::SkillProviderInterfacePrx provider = nullptr;
            //manager::dti::SkillManagerInterfacePrx manager = nullptr;

        protected:
            /// Use conditions during runtime this way
            std::atomic_bool stopped = false;
            std::atomic_bool timeoutReached = false;
        };
    }
}
